/// <reference path="./custom.d.ts" />
// tslint:disable
/**
 * DevCycle Bucketing API
 * Documents the DevCycle Bucketing API which provides and API interface to User Bucketing and for generated SDKs.
 *
 * OpenAPI spec version: 1.0.0
 *
 *
 * NOTE: This file is auto generated by the swagger code generator program.
 * https://github.com/swagger-api/swagger-codegen.git
 * Do not edit the file manually.
 */

import * as url from "url";
import * as nodeFetch from "node-fetch";
import { Configuration } from "./configuration";

const BASE_PATH = "https://bucketing-api.devcycle.com/".replace(/\/+$/, "");

/**
 *
 * @export
 */
export const COLLECTION_FORMATS = {
    csv: ",",
    ssv: " ",
    tsv: "\t",
    pipes: "|",
};

/**
 *
 * @export
 * @interface FetchAPI
 */
export interface FetchAPI {
    (url: string, init?: any): Promise<Response>;
}

/**
 *
 * @export
 * @interface FetchArgs
 */
export interface FetchArgs {
    url: string;
    options: any;
}

/**
 *
 * @export
 * @class BaseAPI
 */
export class BaseAPI {
    protected configuration: Configuration

    constructor(configuration: Configuration, protected basePath: string = BASE_PATH, protected fetch: any = nodeFetch) {
        this.configuration = configuration;
        this.basePath = configuration.basePath || this.basePath;
    }
};

/**
 *
 * @export
 * @class RequiredError
 * @extends {Error}
 */
export class RequiredError extends Error {
    constructor(public field: string, msg?: string) {
        super(msg);
    }
}

/**
 *
 * @export
 * @interface ErrorResponse
 */
export interface ErrorResponse {
    /**
     * Error message
     * @type {string}
     * @memberof ErrorResponse
     */
    message: string;
    /**
     * Additional error information detailing the error reasoning
     * @type {any}
     * @memberof ErrorResponse
     */
    data?: any;
}
/**
 *
 * @export
 * @interface Feature
 */
export interface Feature {
    /**
     * unique database id
     * @type {string}
     * @memberof Feature
     */
    _id: string;
    /**
     * Unique key by Project, can be used in the SDK / API to reference by 'key' rather than _id.
     * @type {string}
     * @memberof Feature
     */
    key: string;
    /**
     * Feature type
     * @type {string}
     * @memberof Feature
     */
    type: Feature.TypeEnum;
    /**
     * Bucketed feature variation
     * @type {string}
     * @memberof Feature
     */
    _variation: string;
    /**
     * Evaluation reasoning
     * @type {string}
     * @memberof Feature
     */
    evalReason?: string;
}

/**
 * @export
 * @namespace Feature
 */
export namespace Feature {
    /**
     * @export
     * @enum {string}
     */
    export enum TypeEnum {
        Release = <any> 'release',
        Refactor = <any> 'refactor',
        Experiment = <any> 'experiment',
        NewFeature = <any> 'new_feature'
    }
}
/**
 *
 * @export
 * @interface InlineResponse200
 */
export interface InlineResponse200 {
    /**
     *
     * @type {string}
     * @memberof InlineResponse200
     */
    message?: string;
}
/**
 *
 * @export
 * @interface UserData
 */
export interface UserData {
    /**
     * Unique id to identify the user
     * @type {string}
     * @memberof UserData
     */
    user_id: string;
    /**
     * User's email used to identify the user on the dashaboard / target audiences
     * @type {string}
     * @memberof UserData
     */
    email?: string;
    /**
     * User's name used to idenify the user on the dashaboard / target audiences
     * @type {string}
     * @memberof UserData
     */
    name?: string;
    /**
     * User's language in ISO 639-1 format
     * @type {string}
     * @memberof UserData
     */
    language?: string;
    /**
     * User's country in ISO 3166 alpha-2 format
     * @type {string}
     * @memberof UserData
     */
    country?: string;
    /**
     * App Version of the running application
     * @type {string}
     * @memberof UserData
     */
    appVersion?: string;
    /**
     * App Build number of the running application
     * @type {string}
     * @memberof UserData
     */
    appBuild?: string;
    /**
     * User's custom data to target the user with, data will be logged to DevCycle for use in dashboard.
     * @type {any}
     * @memberof UserData
     */
    customData?: any;
    /**
     * User's custom data to target the user with, data will not be logged to DevCycle only used for feature bucketing.
     * @type {any}
     * @memberof UserData
     */
    privateCustomData?: any;
    /**
     * Date the user was created, Unix epoch timestamp format
     * @type {number}
     * @memberof UserData
     */
    createdDate?: number;
    /**
     * Date the user was created, Unix epoch timestamp format
     * @type {number}
     * @memberof UserData
     */
    lastSeenDate?: number;
    /**
     * Platform the Client SDK is running on
     * @type {string}
     * @memberof UserData
     */
    platform?: string;
    /**
     * Version of the platform the Client SDK is running on
     * @type {string}
     * @memberof UserData
     */
    platformVersion?: string;
    /**
     * User's device type
     * @type {string}
     * @memberof UserData
     */
    deviceType?: string;
    /**
     * DevCycle SDK type
     * @type {string}
     * @memberof UserData
     */
    sdkType?: UserData.SdkTypeEnum;
    /**
     * DevCycle SDK Version
     * @type {string}
     * @memberof UserData
     */
    sdkVersion?: string;
}

/**
 * @export
 * @namespace UserData
 */
export namespace UserData {
    /**
     * @export
     * @enum {string}
     */
    export enum SdkTypeEnum {
        Api = <any> 'api',
        Server = <any> 'server'
    }
}
/**
 *
 * @export
 * @interface UserDataAndEventBody
 */
export interface UserDataAndEventBody {
    /**
     *
     * @type {Array<any>}
     * @memberof UserDataAndEventBody
     */
    event?: Array<any>;
    /**
     *
     * @type {UserData}
     * @memberof UserDataAndEventBody
     */
    user?: UserData;
}

/**
 *
 * @export
 * @interface Variable
 */
export interface Variable {
    /**
     * unique database id
     * @type {string}
     * @memberof Variable
     */
    _id: string;
    /**
     * Unique key by Project, can be used in the SDK / API to reference by 'key' rather than _id.
     * @type {string}
     * @memberof Variable
     */
    key: string;
    /**
     * Variable type
     * @type {string}
     * @memberof Variable
     */
    type: Variable.TypeEnum;
    /**
     * Variable value can be a string, number, boolean, or JSON
     * @type {any}
     * @memberof Variable
     */
    value: any;
}

/**
 * @export
 * @namespace Variable
 */
export namespace Variable {
    /**
     * @export
     * @enum {string}
     */
    export enum TypeEnum {
        String = <any> 'String',
        Boolean = <any> 'Boolean',
        Number = <any> 'Number',
        JSON = <any> 'JSON'
    }
}
/**
 * DVC - fetch parameter creator
 * @export
 */
export const DefaultApiFetchParamCreator = function (configuration?: Configuration) {
    return {
        /**
         *
         * @summary Get all features by key for user data
         * @param {UserData} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        getFeatures(body: UserData, options: any = {}): FetchArgs {
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body','Required parameter body was null or undefined when calling getFeatures.');
            }
            const localVarPath = `/v1/features`;
            const localVarUrlObj = url.parse(localVarPath, true);
            const localVarRequestOptions = Object.assign({ method: 'POST' }, options);
            const localVarHeaderParameter = {} as any;
            const localVarQueryParameter = {} as any;

            // authentication bearerAuth required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
					? configuration.apiKey("Authorization")
					: configuration.apiKey;
                localVarHeaderParameter["Authorization"] = localVarApiKeyValue;
            }

            localVarHeaderParameter['Content-Type'] = 'application/json';

            localVarUrlObj.query = Object.assign({}, localVarUrlObj.query, localVarQueryParameter, options.query);
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            localVarRequestOptions.headers = Object.assign({}, localVarHeaderParameter, options.headers);
            const needsSerialization = (<any>"UserData" !== "string") || localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.body =  needsSerialization ? JSON.stringify(body || {}) : (body || "");

            return {
                url: url.format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Get variable by key for user data
         * @param {UserData} body
         * @param {string} key Variable key
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        getVariableByKey(body: UserData, key: string, options: any = {}): FetchArgs {
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body','Required parameter body was null or undefined when calling getVariableByKey.');
            }
            // verify required parameter 'key' is not null or undefined
            if (key === null || key === undefined) {
                throw new RequiredError('key','Required parameter key was null or undefined when calling getVariableByKey.');
            }
            const localVarPath = `/v1/variables/{key}`
                .replace(`{${"key"}}`, encodeURIComponent(String(key)));
            const localVarUrlObj = url.parse(localVarPath, true);
            const localVarRequestOptions = Object.assign({ method: 'POST' }, options);
            const localVarHeaderParameter = {} as any;
            const localVarQueryParameter = {} as any;

            // authentication bearerAuth required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
					? configuration.apiKey("Authorization")
					: configuration.apiKey;
                localVarHeaderParameter["Authorization"] = localVarApiKeyValue;
            }

            localVarHeaderParameter['Content-Type'] = 'application/json';

            localVarUrlObj.query = Object.assign({}, localVarUrlObj.query, localVarQueryParameter, options.query);
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            localVarRequestOptions.headers = Object.assign({}, localVarHeaderParameter, options.headers);
            const needsSerialization = (<any>"UserData" !== "string") || localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.body =  needsSerialization ? JSON.stringify(body || {}) : (body || "");

            return {
                url: url.format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Get all variables by key for user data
         * @param {UserData} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        getVariables(body: UserData, options: any = {}): FetchArgs {
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body','Required parameter body was null or undefined when calling getVariables.');
            }
            const localVarPath = `/v1/variables`;
            const localVarUrlObj = url.parse(localVarPath, true);
            const localVarRequestOptions = Object.assign({ method: 'POST' }, options);
            const localVarHeaderParameter = {} as any;
            const localVarQueryParameter = {} as any;

            // authentication bearerAuth required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
					? configuration.apiKey("Authorization")
					: configuration.apiKey;
                localVarHeaderParameter["Authorization"] = localVarApiKeyValue;
            }

            localVarHeaderParameter['Content-Type'] = 'application/json';

            localVarUrlObj.query = Object.assign({}, localVarUrlObj.query, localVarQueryParameter, options.query);
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            localVarRequestOptions.headers = Object.assign({}, localVarHeaderParameter, options.headers);
            const needsSerialization = (<any>"UserData" !== "string") || localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.body =  needsSerialization ? JSON.stringify(body || {}) : (body || "");

            return {
                url: url.format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Post events to DevCycle for user
         * @param {UserDataAndEventBody} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        postEvents(body: UserDataAndEventBody, options: any = {}): FetchArgs {
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body','Required parameter body was null or undefined when calling postEvents.');
            }
            const localVarPath = `/v1/track`;
            const localVarUrlObj = url.parse(localVarPath, true);
            const localVarRequestOptions = Object.assign({ method: 'POST' }, options);
            const localVarHeaderParameter = {} as any;
            const localVarQueryParameter = {} as any;

            // authentication bearerAuth required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
					? configuration.apiKey("Authorization")
					: configuration.apiKey;
                localVarHeaderParameter["Authorization"] = localVarApiKeyValue;
            }

            localVarHeaderParameter['Content-Type'] = 'application/json';

            localVarUrlObj.query = Object.assign({}, localVarUrlObj.query, localVarQueryParameter, options.query);
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            localVarRequestOptions.headers = Object.assign({}, localVarHeaderParameter, options.headers);
            const needsSerialization = (<any>"UserDataAndEventBody" !== "string") || localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.body =  needsSerialization ? JSON.stringify(body || {}) : (body || "");

            return {
                url: url.format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
    }
};

/**
 * DVC - functional programming interface
 * @export
 */
export const DefaultApiFp = function(configuration?: Configuration) {
    return {
        /**
         *
         * @summary Get all features by key for user data
         * @param {UserData} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        getFeatures(body: UserData, options?: any): (fetch?: FetchAPI, basePath?: string) => Promise<{ [key: string]: Feature; }> {
            const localVarFetchArgs = DefaultApiFetchParamCreator(configuration).getFeatures(body, options);
            return (fetch: FetchAPI = nodeFetch, basePath: string = BASE_PATH) => {
                return fetch(basePath + localVarFetchArgs.url, localVarFetchArgs.options).then((response) => {
                    if (response.status >= 200 && response.status < 300) {
                        return response.json();
                    } else {
                        throw response;
                    }
                });
            };
        },
        /**
         *
         * @summary Get variable by key for user data
         * @param {UserData} body
         * @param {string} key Variable key
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        getVariableByKey(body: UserData, key: string, defaultValue: any, options?: any): (fetch?: FetchAPI, basePath?: string) => Promise<Variable> {
            const localVarFetchArgs = DefaultApiFetchParamCreator(configuration).getVariableByKey(body, key, options);
            return (fetch: FetchAPI = nodeFetch, basePath: string = BASE_PATH) => {
                return fetch(basePath + localVarFetchArgs.url, localVarFetchArgs.options).then((response) => {
                    if (response.status >= 200 && response.status < 300) {
                        return response.json();
                    } else {
                        console.error(response)
                        return {
                            value: defaultValue,
                            key
                        }
                    }
                });
            };
        },
        /**
         *
         * @summary Get all variables by key for user data
         * @param {UserData} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        getVariables(body: UserData, options?: any): (fetch?: FetchAPI, basePath?: string) => Promise<{ [key: string]: Variable; }> {
            const localVarFetchArgs = DefaultApiFetchParamCreator(configuration).getVariables(body, options);
            return (fetch: FetchAPI = nodeFetch, basePath: string = BASE_PATH) => {
                return fetch(basePath + localVarFetchArgs.url, localVarFetchArgs.options).then((response) => {
                    if (response.status >= 200 && response.status < 300) {
                        return response.json();
                    } else {
                        throw response;
                    }
                });
            };
        },
        /**
         *
         * @summary Post events to DevCycle for user
         * @param {UserDataAndEventBody} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        postEvents(body: UserDataAndEventBody, options?: any): (fetch?: FetchAPI, basePath?: string) => Promise<InlineResponse200> {
            const localVarFetchArgs = DefaultApiFetchParamCreator(configuration).postEvents(body, options);
            return (fetch: FetchAPI = nodeFetch, basePath: string = BASE_PATH) => {
                return fetch(basePath + localVarFetchArgs.url, localVarFetchArgs.options).then((response) => {
                    if (response.status >= 200 && response.status < 300) {
                        return response.json();
                    } else {
                        throw response;
                    }
                });
            };
        },
    }
};

/**
 * DVC - factory interface
 * @export
 */
export const DefaultApiFactory = function (configuration?: Configuration, fetch?: FetchAPI, basePath?: string) {
    return {
        /**
         *
         * @summary Get all features by key for user data
         * @param {UserData} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        getFeatures(body: UserData, options?: any) {
            return DefaultApiFp(configuration).getFeatures(body, options)(fetch, basePath);
        },
        /**
         *
         * @summary Get variable by key for user data
         * @param {UserData} body
         * @param {string} key Variable key
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        getVariableByKey(body: UserData, key: string, defaultValue: any, options?: any) {
            return DefaultApiFp(configuration).getVariableByKey(body, key, defaultValue, options)(fetch, basePath);
        },
        /**
         *
         * @summary Get all variables by key for user data
         * @param {UserData} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        getVariables(body: UserData, options?: any) {
            return DefaultApiFp(configuration).getVariables(body, options)(fetch, basePath);
        },
        /**
         *
         * @summary Post events to DevCycle for user
         * @param {UserDataAndEventBody} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        postEvents(body: UserDataAndEventBody, options?: any) {
            return DefaultApiFp(configuration).postEvents(body, options)(fetch, basePath);
        },
    };
};

/**
 * DVC - object-oriented interface
 * @export
 * @class DVC
 * @extends {BaseAPI}
 */
export class DVC extends BaseAPI {

    /**
     *
     * @summary Get all features by key for user data
     * @param {UserData} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof DVC
     */
    public getFeatures(body: UserData, options?: any) {
        return DefaultApiFp(this.configuration).getFeatures(body, options)(this.fetch, this.basePath);
    }

    /**
     *
     * @summary Get variable by key for user data
     * @param {UserData} body
     * @param {string} key Variable key
     * @param {string} defaultValue Value returned if request to worker fails or variable not served to user
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof DVC
     */
    public getVariableByKey(body: UserData, key: string, defaultValue: any, options?: any) {
        return DefaultApiFp(this.configuration).getVariableByKey(body, key, defaultValue, options)(this.fetch, this.basePath);
    }

    /**
     *
     * @summary Get all variables by key for user data
     * @param {UserData} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof DVC
     */
    public getVariables(body: UserData, options?: any) {
        return DefaultApiFp(this.configuration).getVariables(body, options)(this.fetch, this.basePath);
    }

    /**
     *
     * @summary Post events to DevCycle for user
     * @param {UserDataAndEventBody} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof DVC
     */
    public postEvents(body: UserDataAndEventBody, options?: any) {
        return DefaultApiFp(this.configuration).postEvents(body, options)(this.fetch, this.basePath);
    }

}
